require_dependency "nukleus/authenticated_controller"

module Nukleus
    class DocumentsController < AuthenticatedController
        before_action :set_document, only: [:show, :edit, :update, :destroy]

        def index
            @documents = Document.all
        end

        def show
        end

        def new
            @document = Document.new
        end

        def edit
        end

        def create
            @document = Document.new(document_params)
            respond_to do |format|
                if @document.save
                    format.json { render :show, status: :created }
                else
                    format.json { render json: @document.errors, status: :unprocessable_entity }
                end
            end
        end

        def update
            @document = @document.update_with_copy(document_params)
            if @document.errors.messages.keys.count == 0
                render :show, status: :ok
            else
                render json: @document.errors, status: :unprocessable_entity
            end
        end

        def destroy
            @document.destroy
            head :no_content
        end

        private

        def set_document
            @document = Document.where(document_id: params[:id]).first
        end

        def document_params
            # The meta_data is an undefined data structure. Therefore, we cannot use the permit call below.
            meta_data = params["document"]["meta_data"]
            tags = params["document"]["tags"]
            params["document"].delete("meta_data")
            params["document"].delete("tags")
            changed_params = params.require(:document).permit(:document_file, :document_id, :document_file_filename)
            changed_params["meta_data"] = meta_data unless meta_data.nil?
            changed_params["tags"] = tags unless tags.nil?

            changed_params
        end
    end
end
