require 'rails_helper'
module Nukleus
    RSpec.describe DocumentsController, type: :controller do
        before(:example) do
            NoBrainer.purge!
            NoBrainer.sync_indexes
        end

        let(:document) { Document.new({:document_id =>"test_doc_id", :tags => ["a","b"], :meta_data => { test: "me" }}) }

        # describe "print routes" do
        #   it "test" do
        #     test_routes = []

        #     Nukleus::Engine.routes.routes.each do |route|
        #       route = route.path.spec.to_s
        #       test_routes << route
        #     end
        #     puts test_routes
        #   end
        # end

        describe "GET #index" do
            routes { ::Nukleus::Engine.routes }
            render_views

            it "returns http success" do
                get :index, :format => :json
                expect(response).to have_http_status(:success)
            end
        end

        describe "GET #show" do
            routes { ::Nukleus::Engine.routes }
            render_views

            before(:each) do
                FactoryGirl.create(:nukleus_document)
            end

            it "returns http success" do
                get :show, id: "factory_doc_id", :format => :json
                expect(response).to have_http_status(:success)
            end
        end

        describe "POST #create" do
            routes { ::Nukleus::Engine.routes }
            render_views

            it "returns http success using json" do
                expect {
                    post :create, document: { document_id: "test", tags: ["bla", "blubb"], meta_data: { test: "me", follow: "you"} }, :format => :json
                }.to change { Document.count }
                expect(response).to have_http_status(:created)

                resp = JSON.parse(response.body)

                # TODO: Make this possible with new structure: expect(resp["token"]).not_to eq(nil)
                expect(resp["data"]).not_to eq(nil)
            end

        end

        describe "PUT #update" do
            routes { ::Nukleus::Engine.routes }

            before(:each) do
                FactoryGirl.create(:nukleus_document)
            end

            it "returns http success" do
                put :update, id: "factory_doc_id", document: { document_id: "test", tags: ["bla", "blubb"], meta_data: {test: "me", follow: "you"} }, :format => :json
                expect(response).to have_http_status(:success)
            end
        end

        describe "PATCH #update" do
            routes { ::Nukleus::Engine.routes }
            render_views

            before(:each) do
                FactoryGirl.create(:nukleus_document)
                patch :update, id: "factory_doc_id", document: { document_id: "test", tags: ["bla", "blubb"], meta_data: {test: "me", follow: "you"} }, :format => :json
            end

            it "returns http success" do
                expect(response).to have_http_status(:success)
            end

            it "exists two documents" do
                expect(Document.where(document_id:"factory_doc_id").count).to eq(2)
            end

            it "increases the version number" do
                # Check whether the returned document is the new version
                response_object = JSON.parse(response.body)
                expect(response_object["data"]["document_version"]).to eq(1)
            end

            it "changed the data" do
                response_object = JSON.parse(response.body)
                expect(response_object["data"]["tags"]).to eq(["bla", "blubb"])
            end
        end

        describe "delete #destroy" do
            routes { ::Nukleus::Engine.routes }

            before(:each) do
                FactoryGirl.create(:nukleus_document)
            end

            it "returns http success" do
                delete :destroy, id: "factory_doc_id", :format => :json
                expect(response).to have_http_status(:success)
            end

            it "removes all versions" do
                # First create 2 versions
                patch :update, id: "factory_doc_id", document: { document_id: "test", tags: ["bla", "blubb"], meta_data: {test: "me", follow: "you"} }, :format => :json

                # Check whether we really have two versions
                expect(Document.where(document_id:"factory_doc_id").count).to eq(2)

                # Remove one version
                Document.first.destroy

                expect(Document.where(document_id:"factory_doc_id").count).to eq(0)

            end
        end

    end
end
